#!perl -w

use Win32;
use File::Find;
use Getopt::Std;
use strict;

use vars qw( %opts );

# ----------------------------------------------------------------------------

sub usage
{
<<EOF;
usage: $0 -w c:/path/to/winamp.exe -o outputfile dir1 [dir2 ...]

Locates all mp# files in all subdirectories given as arguments, randomizes
them, and creates a playlist.m3u file in C:/temp, or the directory specified
by -o. This file is then sent to WinAmp to be played. If -w is not specified,
WinAmp is not executed. Use forward slashes, and put double quotes around
items with spaces.

example: $0 -w "c:/Program Files/WinAmp/winamp.exe" e:/mp3 f:/mp3

PlayRand version 0.1
http://www.cs.virginia.edu/~dwc3q/code/index.html
EOF
}

# ----------------------------------------------------------------------------

sub RandomizeArray
{
my @unshuffled = @_;
my @shuffled = ();

srand;

for(@unshuffled)
{
  my $place = rand @shuffled+1;
  push(@shuffled,$shuffled[$place]);
  $shuffled[$place] = $_;
}

@shuffled;
}

# ----------------------------------------------------------------------------

print usage and exit if $#ARGV == -1 or $ARGV[0] =~ /^-h/i;

getopt('ow',\%opts);
$opts{o} = 'c:/temp/playlist.m3u' if !defined $opts{o};

my @songs;

while (my $directory = shift @ARGV)
{
  # Gets all mp3 files in current directory and descendents. Puts them in
  # @songs
  find(sub {push @songs,"$File::Find::name" if /\.mp\d+$/i},$directory);
}

@songs = RandomizeArray @songs;

$, = "\n";
open M3U, ">$opts{o}";
print M3U @songs,"\n";
close M3U;

if (exists $opts{w})
{
  my $winamp = Win32::GetShortPathName $opts{w};
  $winamp =~ s/\0.*//; 

  print "$opts{w} not found.\n" and exit if $winamp eq '';

  exec "$winamp $opts{o}";
}
